==> bird-0.0.1%2B3.tar.gz: test/smart_api/util/optional_parsed_result_test.dart <==
import 'package:bird/smart_api.dart';
import 'package:dartz/dartz.dart';
import 'package:stack_trace/stack_trace.dart';
import 'package:test/test.dart';

void main() {
  group("$StringConversion", () {
    test("basic", () {
      final StringConversion c = StringConversion("[1, 2, 3]");

      /// Raw string.
      expect(c.str, "[1, 2, 3]");

      /// First call converts it to json.
      expect(c.cachedJson(), [1, 2, 3]);

      /// Second call returns the previous conversion.
      expect(c.cachedJson(), [1, 2, 3]);
    });

    test("==", () =>
        expect(StringConversion("[1, 2, 3]"), StringConversion("[1, 2, 3]")));

    test("!=", () =>
        expect(StringConversion("[1, 2, 3]"),
            isNot(StringConversion("[1, 2, 4]"))));

    test("hashCode", () =>
        expect(StringConversion("[1, 2, 3]").hashCode,
            StringConversion("[1, 2, 3]").hashCode));

    test("apply", () {
      int x;
      expect(StringConversion("aaaa").apply<void>((a) => x = a.str.length),
          StringConversion("aaaa"));
      expect(x, 4);
    });
  });

  group("$ParsingError", () {
    test("map", () {
      /// Subclasses test the rest
      expect(() => _TestMyParsingError<bool>().map((_) => false, (_) => false),
          throwsA(const TypeMatcher<Exception>()));
    });
  });

  group("$ThrowsWhileParsingError", () {
    test("==", () {
      // ignore: prefer_const_constructors
      expect(const ThrowsWhileParsingError<String>(
          null, null, ThrowsAtStage.preInterestingCheck, null, ""),
          const ThrowsWhileParsingError<String>(
              null, null, ThrowsAtStage.preInterestingCheck, "", ""));
      expect(const ThrowsWhileParsingError<String>(
          null, null, ThrowsAtStage.preInterestingCheck, "", ""),
          ThrowsWhileParsingError<String>(
              null, Trace.current(), ThrowsAtStage.preInterestingCheck, "",
              ""));
    });

    test("!=", () {
      expect(const ThrowsWhileParsingError<String>(
          null, null, ThrowsAtStage.preInterestingCheck, "", ""),
          isNot(const ThrowsWhileParsingError<String>(
              "", null, ThrowsAtStage.parsing, "", "")));
      expect(const ThrowsWhileParsingError<String>(
          null, null, ThrowsAtStage.preInterestingCheck, "", ""),
          isNot(ThrowsWhileParsingError<String>(
              "", StackTrace.fromString(""), ThrowsAtStage.preInterestingCheck,
              "", "")));
    });

    test("hashCode", () {
      expect(const ThrowsWhileParsingError<String>(
          null, null, ThrowsAtStage.preInterestingCheck, "-", "").hashCode,
          const ThrowsWhileParsingError<String>(
              null, null, ThrowsAtStage.preInterestingCheck, "11", "")
              .hashCode);
    });

    test("map", () {
      expect(const ThrowsWhileParsingError<String>(
          null, null, ThrowsAtStage.preInterestingCheck, "", "").map(
            (_) => false,
            (_) => true,
      ), true);
    });

    test("toString", () {
      expect(const ThrowsWhileParsingError<String>(
          null, null, ThrowsAtStage.preInterestingCheck, "", "").toString(),
          contains("$ThrowsWhileParsingError"));
    });
  });

  group("$ParsingValidationError", () {
    test("==", () {
      // ignore: prefer_const_constructors
      expect(const ParsingValidationError(null, null),
          // ignore: prefer_const_constructors
          ParsingValidationError(null, null));
      expect(const ParsingValidationError<String>(null, null),
          ParsingValidationError<String>(
              ValidationCheck<String>((String a) => true, "bla"), null));
    });

    test("!=", () {
      // ignore: prefer_const_constructors
      expect(const ParsingValidationError(null, null),
          // ignore: prefer_const_constructors
          isNot(ParsingValidationError(null, "a")));
      expect(const ParsingValidationError<String>(null, ""), isNot(
          const ThrowsWhileParsingError<String>(
              null, null, ThrowsAtStage.preInterestingCheck, "", "")));
    });

    test("hashCode", () {
      expect(const ParsingValidationError(null, null).hashCode,
          const ParsingValidationError(null, null).hashCode);
    });

    test("map", () {
      expect(const ParsingValidationError(null, null).map(
            (_) => true,
            (_) => false,
      ), true);
    });

    test("toString", () {
      expect(const ParsingValidationError(null, "a").toString(),
          startsWith("$ParsingValidationError"));
    });
  });

  group("$ValidationCheck", () {
    test("toString", () {
      expect(
          ValidationCheck<dynamic>((dynamic a) => true, "check 1").toString(),
          startsWith("$ValidationCheck"));
    });
  });

  group("$PreInterestCheck", () {
    test("toString", () {
      expect(PreInterestCheck((a) => true, "check 1").toString(),
          startsWith("$PreInterestCheck"));
    });
  });

  group("$PostInterestCheck", () {
    test("toString", () {
      expect(
          PostInterestCheck<dynamic>((dynamic a) => true, "check 1").toString(),
          startsWith("$PostInterestCheck"));
    });
  });

  group("$OptionalParsedResult", () {
    group("constructor", () {
      test("fail preInterestinng", () {
        expect(
          OptionalParsedResult<String>(
            source: StringConversion("a"),
            converter: (c) => c.str,
            preInteresting: [
              PreInterestCheck((_) => false, "")
            ],
          ).value,
          const None<dynamic>(),
        );
      });

      test("fail tryConversion", () {
        final e = Exception("-");
        expect(
          OptionalParsedResult<String>(
            source: StringConversion("a"),
            converter: (c) => throw e,
          ).value,
          Some(Left<dynamic, dynamic>(ThrowsWhileParsingError<String>(
              e, null, ThrowsAtStage.parsing, "", ""))),
        );
      });

      test("fail tryPostInteresting", () {
        expect(
          OptionalParsedResult<String>(
            source: StringConversion("a"),
            converter: (c) => c.str,
            postInteresting: [
              PostInterestCheck((_) => false, "")
            ],
          ).value,
          const None<dynamic>(),
        );
      });

      test("fail tryValidation", () {
        expect(OptionalParsedResult<String>(
          source: StringConversion("a"),
          converter: (c) => "aaa",
          validations: [
            ValidationCheck((_) => false, "test"),
          ],
        ).value, const Some(
            Left<dynamic, dynamic>(ParsingValidationError(null, "aaa"))));
      });

      test("succeed parsing", () {
        expect(OptionalParsedResult<String>(
          source: StringConversion("a"),
          converter: (c) => "aaa",
          validations: [
            ValidationCheck((_) => true, "test"),
          ],
        ).value, const Some(Right<dynamic, String>("aaa")));
      });
    });

    test("constructor.none", () {
      expect(const OptionalParsedResult<dynamic>.none().value, none<dynamic>());
    });

    group("${OptionalParsedResult.tryPreInteresting}", () {
      test("last fails", () {
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => true, "fail1"),
          PreInterestCheck((_) => true, "fail2"),
          PreInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("first fails", () {
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => false, "fail1"),
          PreInterestCheck((_) => true, "fail2"),
          PreInterestCheck((_) => true, "fail3"),
        ]), const None<dynamic>());
      });

      test("all fail", () {
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => false, "fail1"),
          PreInterestCheck((_) => false, "fail2"),
          PreInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("doesn't fail", () {
        expect(OptionalParsedResult.tryPreInteresting<dynamic>(
            StringConversion("aa"), [
          PreInterestCheck((_) => true, "fail1"),
          PreInterestCheck((_) => true, "fail2"),
          PreInterestCheck((_) => true, "fail3"),
        ]), null);
      });

      test("throws", () {
        final e = Exception("-");
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => throw e, "fail1"),
        ]), Some<Either<ParsingError<String>, String>>(
            Left<ParsingError<String>, String>(ThrowsWhileParsingError<String>(
                e, null, ThrowsAtStage.preInterestingCheck, "fail1", ""))));
      });
    });

    group("${OptionalParsedResult.tryConversion}", () {
      test("converts successfully", () {
        expect(OptionalParsedResult.tryConversion<String>(() => "a", ""),
            const Left<dynamic, dynamic>("a"));
      });

      test("throws", () {
        final e = Exception("e");
        expect(OptionalParsedResult.tryConversion<String>(() => throw e, ""),
            Right<dynamic, dynamic>(Some(Left<dynamic, dynamic>(
                ThrowsWhileParsingError<String>(
                    e, null, ThrowsAtStage.parsing, null, "")))));
      });
    });

    group("${OptionalParsedResult.tryPostInteresting}", () {
      test("last fails", () {
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck((_) => true, "fail1"),
          PostInterestCheck((_) => true, "fail2"),
          PostInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("first fails", () {
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck((_) => false, "fail1"),
          PostInterestCheck((_) => true, "fail2"),
          PostInterestCheck((_) => true, "fail3"),
        ]), const None<dynamic>());
      });

      test("all fail", () {
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck((_) => false, "fail1"),
          PostInterestCheck((_) => false, "fail2"),
          PostInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("doesn't fail", () {
        expect(OptionalParsedResult.tryPostInteresting<dynamic>("aa", [
          PostInterestCheck<dynamic>((dynamic _) => true, "fail1"),
          PostInterestCheck<dynamic>((dynamic _) => true, "fail2"),
          PostInterestCheck<dynamic>((dynamic _) => true, "fail3"),
        ]), null);
      });

      test("throws", () {
        final e = Exception("-");
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck<String>((String _) => throw e, "fail1"),
        ]), Some<Either<ParsingError<String>, String>>(
            Left<ParsingError<String>, String>(
                ThrowsWhileParsingError<String>(
                    e, null, ThrowsAtStage.postInterestingCheck, "fail1",
                    ""))));
      });
    });

    group("${OptionalParsedResult.tryValidation}", () {
      test("last fails", () {
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck((_) => true, "fail1"),
          ValidationCheck((_) => true, "fail2"),
          ValidationCheck((_) => false, "fail3"),
        ]), Some(Left<dynamic, dynamic>(ParsingValidationError<String>(
            ValidationCheck<String>((String _) => false, "fail3"), "aa"))));
      });

      test("first fails", () {
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck<String>((_) => false, "fail1"),
          ValidationCheck<String>((_) => true, "fail2"),
          ValidationCheck<String>((_) => true, "fail3"),
        ]), Some(
            Left<ParsingError<String>, String>(ParsingValidationError<String>(
                ValidationCheck<String>((String _) => false, "fail1"), "aa"))));
      });

      test("all fail", () {
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck((_) => false, "fail1"),
          ValidationCheck((_) => false, "fail2"),
          ValidationCheck((_) => false, "fail3"),
        ]), Some(Left<dynamic, dynamic>(ParsingValidationError<String>(
            ValidationCheck<String>((String _) => false, "fail1"), "aa"))));
      });

      test("doesn't fail", () {
        expect(OptionalParsedResult.tryValidation<dynamic>("aa", [
          ValidationCheck<dynamic>((dynamic _) => true, "fail1"),
          ValidationCheck<dynamic>((dynamic _) => true, "fail2"),
          ValidationCheck<dynamic>((dynamic _) => true, "fail3"),
        ]), const Some(Right<dynamic, dynamic>("aa")));
      });

      test("throws", () {
        final e = Exception("-");
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck<String>((String _) => throw e, "fail1"),
        ]), Some(
            Left<ParsingError<String>, String>(ThrowsWhileParsingError<String>(
                e, null, ThrowsAtStage.validation, "fail1", ""))));
      });
    });

    test("toOption", () {
      expect(OptionalParsedResult<String>(
        source: StringConversion("a"),
        converter: (c) => c.str,
        preInteresting: [
          PreInterestCheck((_) => false, "")
        ],
      ).value, const None<dynamic>());

      expect(OptionalParsedResult<String>(
        source: StringConversion("a"),
        converter: (c) => c.str,
        preInteresting: [
        ],
      ).value, const Some(Right<String, String>("a")));
    });

    test("==", () {
      expect(OptionalParsedResult<String>.raw(some(right(""))),
          OptionalParsedResult<String>.raw(some(right(""))));
    });

    test("!=", () {
      expect(OptionalParsedResult<String>.raw(some(right(""))),
          isNot(OptionalParsedResult<dynamic>.raw(some(
              left<ParsingError, dynamic>(const ThrowsWhileParsingError<String>(
                  null, null, ThrowsAtStage.preInterestingCheck, "", ""))))));
    });

    test("hashCode", () {
      expect(OptionalParsedResult<String>.raw(some(right(""))).hashCode,
          OptionalParsedResult<String>
              .raw(some(right("")))
              .hashCode);
    });
    test("toString", () {
      expect(OptionalParsedResult<dynamic>.raw(none()).toString(),
          startsWith("$OptionalParsedResult"));
    });
  });
}

class _TestMyParsingError<T> extends ParsingError<T> {}

==> bird-0.0.3%2B3.tar.gz: test/smart_api/util/optional_parsed_result_test.dart <==
import 'package:bird/smart_api.dart';
import 'package:dartz/dartz.dart';
import 'package:stack_trace/stack_trace.dart';
import 'package:test/test.dart';

void main() {
  group("$StringConversion", () {
    test("basic", () {
      final StringConversion c = StringConversion("[1, 2, 3]");

      /// Raw string.
      expect(c.str, "[1, 2, 3]");

      /// First call converts it to json.
      expect(c.cachedJson(), [1, 2, 3]);

      /// Second call returns the previous conversion.
      expect(c.cachedJson(), [1, 2, 3]);
    });

    test("==", () =>
        expect(StringConversion("[1, 2, 3]"), StringConversion("[1, 2, 3]")));

    test("!=", () =>
        expect(StringConversion("[1, 2, 3]"),
            isNot(StringConversion("[1, 2, 4]"))));

    test("hashCode", () =>
        expect(StringConversion("[1, 2, 3]").hashCode,
            StringConversion("[1, 2, 3]").hashCode));

    test("apply", () {
      int x;
      expect(StringConversion("aaaa").apply<void>((a) => x = a.str.length),
          StringConversion("aaaa"));
      expect(x, 4);
    });
  });

  group("$ThrowsWhileParsingError", () {
    test("==", () {
      // ignore: prefer_const_constructors
      expect(const ThrowsWhileParsingError(
          null, null, ThrowsAtStage.preInterestingCheck, null, ""),
          const ThrowsWhileParsingError(
              null, null, ThrowsAtStage.preInterestingCheck, "", ""));
      expect(const ThrowsWhileParsingError(
          null, null, ThrowsAtStage.preInterestingCheck, "", ""),
          ThrowsWhileParsingError(
              null, Trace.current(), ThrowsAtStage.preInterestingCheck, "",
              ""));
    });

    test("!=", () {
      expect(const ThrowsWhileParsingError(
          null, null, ThrowsAtStage.preInterestingCheck, "", ""),
          isNot(const ThrowsWhileParsingError(
              "", null, ThrowsAtStage.parsing, "", "")));
      expect(const ThrowsWhileParsingError(
          null, null, ThrowsAtStage.preInterestingCheck, "", ""),
          isNot(ThrowsWhileParsingError(
              "", StackTrace.fromString(""), ThrowsAtStage.preInterestingCheck,
              "", "")));
    });

    test("hashCode", () {
      expect(const ThrowsWhileParsingError(
          null, null, ThrowsAtStage.preInterestingCheck, "-", "").hashCode,
          const ThrowsWhileParsingError(
              null, null, ThrowsAtStage.preInterestingCheck, "11", "")
              .hashCode);
    });

    test("map", () {
      expect(const ThrowsWhileParsingError(
          null, null, ThrowsAtStage.preInterestingCheck, "", "").map(
          parsingValidation: (_) => false,
        throwsWhileParsing: (_) => true,
      ), true);
    });

    test("toString", () {
      expect(const ThrowsWhileParsingError(
          null, null, ThrowsAtStage.preInterestingCheck, "", "").toString(),
          contains("$ThrowsWhileParsingError"));
    });
  });

  group("$ParsingValidationError", () {
    test("==", () {
      // ignore: prefer_const_constructors
      expect(const ParsingValidationError(null, null),
          // ignore: prefer_const_constructors
          ParsingValidationError(null, null));
      expect(const ParsingValidationError<String>(null, null),
          ParsingValidationError<String>(
              ValidationCheck<String>((String a) => true, "bla"), null));
    });

    test("!=", () {
      // ignore: prefer_const_constructors
      expect(const ParsingValidationError(null, null),
          // ignore: prefer_const_constructors
          isNot(ParsingValidationError(null, "a")));
      expect(const ParsingValidationError<String>(null, ""), isNot(
          const ThrowsWhileParsingError(
              null, null, ThrowsAtStage.preInterestingCheck, "", "")));
    });

    test("hashCode", () {
      expect(const ParsingValidationError(null, null).hashCode,
          const ParsingValidationError(null, null).hashCode);
    });

    test("map", () {
      expect(const ParsingValidationError(null, null).map(
        parsingValidation: (_) => true,
        throwsWhileParsing: (_) => false,
      ), true);
    });

    test("toString", () {
      expect(const ParsingValidationError(null, "a").toString(),
          startsWith("$ParsingValidationError"));
    });
  });

  group("$ValidationCheck", () {
    test("toString", () {
      expect(
          ValidationCheck<dynamic>((dynamic a) => true, "check 1").toString(),
          startsWith("$ValidationCheck"));
    });
  });

  group("$PreInterestCheck", () {
    test("toString", () {
      expect(PreInterestCheck((a) => true, "check 1").toString(),
          startsWith("$PreInterestCheck"));
    });
  });

  group("$PostInterestCheck", () {
    test("toString", () {
      expect(
          PostInterestCheck<dynamic>((dynamic a) => true, "check 1").toString(),
          startsWith("$PostInterestCheck"));
    });
  });

  group("$OptionalParsedResult", () {
    group("constructor", () {
      test("fail preInterestinng", () {
        expect(
          OptionalParsedResult<String>(
            source: StringConversion("a"),
            converter: (c) => c.str,
            preInteresting: [
              PreInterestCheck((_) => false, "")
            ],
          ).value,
          const None<dynamic>(),
        );
      });

      test("fail tryConversion", () {
        final e = Exception("-");
        expect(
          OptionalParsedResult<String>(
            source: StringConversion("a"),
            converter: (c) => throw e,
          ).value,
          Some(Left<dynamic, dynamic>(ThrowsWhileParsingError(
              e, null, ThrowsAtStage.parsing, "", ""))),
        );
      });

      test("fail tryPostInteresting", () {
        expect(
          OptionalParsedResult<String>(
            source: StringConversion("a"),
            converter: (c) => c.str,
            postInteresting: [
              PostInterestCheck((_) => false, "")
            ],
          ).value,
          const None<dynamic>(),
        );
      });

      test("fail tryValidation", () {
        expect(OptionalParsedResult<String>(
          source: StringConversion("a"),
          converter: (c) => "aaa",
          validations: [
            ValidationCheck((_) => false, "test"),
          ],
        ).value, const Some(
            Left<dynamic, dynamic>(ParsingValidationError(null, "aaa"))));
      });

      test("succeed parsing", () {
        expect(OptionalParsedResult<String>(
          source: StringConversion("a"),
          converter: (c) => "aaa",
          validations: [
            ValidationCheck((_) => true, "test"),
          ],
        ).value, const Some(Right<dynamic, String>("aaa")));
      });
    });

    test("constructor.none", () {
      expect(const OptionalParsedResult<dynamic>.none().value, none<dynamic>());
    });

    group("${OptionalParsedResult.tryPreInteresting}", () {
      test("last fails", () {
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => true, "fail1"),
          PreInterestCheck((_) => true, "fail2"),
          PreInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("first fails", () {
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => false, "fail1"),
          PreInterestCheck((_) => true, "fail2"),
          PreInterestCheck((_) => true, "fail3"),
        ]), const None<dynamic>());
      });

      test("all fail", () {
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => false, "fail1"),
          PreInterestCheck((_) => false, "fail2"),
          PreInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("doesn't fail", () {
        expect(OptionalParsedResult.tryPreInteresting<dynamic>(
            StringConversion("aa"), [
          PreInterestCheck((_) => true, "fail1"),
          PreInterestCheck((_) => true, "fail2"),
          PreInterestCheck((_) => true, "fail3"),
        ]), null);
      });

      test("throws", () {
        final e = Exception("-");
        expect(OptionalParsedResult.tryPreInteresting<String>(
            StringConversion("aa"), [
          PreInterestCheck((_) => throw e, "fail1"),
        ]), Some<Either<ParsingError, String>>(
            Left<ParsingError, String>(ThrowsWhileParsingError(
                e, null, ThrowsAtStage.preInterestingCheck, "fail1", ""))));
      });
    });

    group("${OptionalParsedResult.tryConversion}", () {
      test("converts successfully", () {
        expect(OptionalParsedResult.tryConversion<String>(() => "a", ""),
            const Left<dynamic, dynamic>("a"));
      });

      test("throws", () {
        final e = Exception("e");
        expect(OptionalParsedResult.tryConversion<String>(() => throw e, ""),
            Right<dynamic, dynamic>(Some(Left<dynamic, dynamic>(
                ThrowsWhileParsingError(
                    e, null, ThrowsAtStage.parsing, null, "")))));
      });
    });

    group("${OptionalParsedResult.tryPostInteresting}", () {
      test("last fails", () {
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck((_) => true, "fail1"),
          PostInterestCheck((_) => true, "fail2"),
          PostInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("first fails", () {
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck((_) => false, "fail1"),
          PostInterestCheck((_) => true, "fail2"),
          PostInterestCheck((_) => true, "fail3"),
        ]), const None<dynamic>());
      });

      test("all fail", () {
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck((_) => false, "fail1"),
          PostInterestCheck((_) => false, "fail2"),
          PostInterestCheck((_) => false, "fail3"),
        ]), const None<dynamic>());
      });

      test("doesn't fail", () {
        expect(OptionalParsedResult.tryPostInteresting<dynamic>("aa", [
          PostInterestCheck<dynamic>((dynamic _) => true, "fail1"),
          PostInterestCheck<dynamic>((dynamic _) => true, "fail2"),
          PostInterestCheck<dynamic>((dynamic _) => true, "fail3"),
        ]), null);
      });

      test("throws", () {
        final e = Exception("-");
        expect(OptionalParsedResult.tryPostInteresting<String>("aa", [
          PostInterestCheck<String>((String _) => throw e, "fail1"),
        ]), Some<Either<ParsingError, String>>(
            Left<ParsingError, String>(
                ThrowsWhileParsingError(
                    e, null, ThrowsAtStage.postInterestingCheck, "fail1",
                    ""))));
      });
    });

    group("${OptionalParsedResult.tryValidation}", () {
      test("last fails", () {
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck((_) => true, "fail1"),
          ValidationCheck((_) => true, "fail2"),
          ValidationCheck((_) => false, "fail3"),
        ]), Some(Left<dynamic, dynamic>(ParsingValidationError<String>(
            ValidationCheck<String>((String _) => false, "fail3"), "aa"))));
      });

      test("first fails", () {
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck<String>((_) => false, "fail1"),
          ValidationCheck<String>((_) => true, "fail2"),
          ValidationCheck<String>((_) => true, "fail3"),
        ]), Some(
            Left<ParsingError, String>(ParsingValidationError<String>(
                ValidationCheck<String>((String _) => false, "fail1"), "aa"))));
      });

      test("all fail", () {
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck((_) => false, "fail1"),
          ValidationCheck((_) => false, "fail2"),
          ValidationCheck((_) => false, "fail3"),
        ]), Some(Left<dynamic, dynamic>(ParsingValidationError<String>(
            ValidationCheck<String>((String _) => false, "fail1"), "aa"))));
      });

      test("doesn't fail", () {
        expect(OptionalParsedResult.tryValidation<dynamic>("aa", [
          ValidationCheck<dynamic>((dynamic _) => true, "fail1"),
          ValidationCheck<dynamic>((dynamic _) => true, "fail2"),
          ValidationCheck<dynamic>((dynamic _) => true, "fail3"),
        ]), const Some(Right<dynamic, dynamic>("aa")));
      });

      test("throws", () {
        final e = Exception("-");
        expect(OptionalParsedResult.tryValidation<String>("aa", [
          ValidationCheck<String>((String _) => throw e, "fail1"),
        ]), Some(
            Left<ParsingError, String>(ThrowsWhileParsingError(
                e, null, ThrowsAtStage.validation, "fail1", ""))));
      });
    });

    test("toOption", () {
      expect(OptionalParsedResult<String>(
        source: StringConversion("a"),
        converter: (c) => c.str,
        preInteresting: [
          PreInterestCheck((_) => false, "")
        ],
      ).value, const None<dynamic>());

      expect(OptionalParsedResult<String>(
        source: StringConversion("a"),
        converter: (c) => c.str,
        preInteresting: [
        ],
      ).value, const Some(Right<String, String>("a")));
    });

    test("==", () {
      expect(OptionalParsedResult<String>.raw(some(right(""))),
          OptionalParsedResult<String>.raw(some(right(""))));
    });

    test("!=", () {
      expect(OptionalParsedResult<String>.raw(some(right(""))),
          isNot(OptionalParsedResult<dynamic>.raw(some(
              left<ParsingError, dynamic>(const ThrowsWhileParsingError(
                  null, null, ThrowsAtStage.preInterestingCheck, "", ""))))));
    });

    test("hashCode", () {
      expect(OptionalParsedResult<String>.raw(some(right(""))).hashCode,
          OptionalParsedResult<String>
              .raw(some(right("")))
              .hashCode);
    });
    test("toString", () {
      expect(OptionalParsedResult<dynamic>.raw(none()).toString(),
          startsWith("$OptionalParsedResult"));
    });
  });
}
